/*! @name videojs-auto-caption @version 0.1.0 @license Apache-2.0 */
import videojs from 'video.js';

var version = "0.1.0";

var defaults = {}; // Cross-compatibility for Video.js 5 and 6.

var registerPlugin = videojs.registerPlugin || videojs.plugin; // const dom = videojs.dom || videojs;

/**
 * Enables the best matching caption
 *
 * @function update
 */

var update = function update() {
  var lang = this.language();
  var tracks = this.textTracks();
  var matches = {
    exact: [],
    twoletter: [],
    fallback: [],
    default: []
  };
  var candidateTracks = Array.prototype.filter.call(tracks, function (t) {
    return t.kind === 'subtitles' || t.kind === 'captions';
  });
  candidateTracks.forEach(function (t) {
    // Player normalises language to lower case
    var trackLang = t.language.toLowerCase(); // `en-US` ~= `en` ~= `en-GB`

    if (trackLang.split('-')[0] === lang.split('-')[0]) {
      matches.twoletter.push(t); // `en-US` === `en-US`

      if (trackLang === lang) {
        matches.exact.push(t);
      }
    } // Honour the default if a language didn't match


    if (t.default) {
      matches.default.push(t);
    }
  }); // Join arrays to order preference

  var selectedTracks = matches.exact.concat(matches.twoletter).concat(matches.defaultMatches);

  if (selectedTracks.length > 0) {
    var selectedTrack = selectedTracks[0];

    for (var i = 0; i < tracks.length; i++) {
      if (tracks[i] === selectedTrack) {
        tracks[i].mode = 'showing';
      } else {
        tracks[i].mode = 'disabled';
      }
    }
  }
};
/**
 * Function to invoke when the player is ready.
 *
 * This is a great place for your plugin to initialize itself. When this
 * function is called, the player will have its DOM and child components
 * in place.
 *
 * @function onPlayerReady
 * @param    {Player} player
 *           A Video.js player object.
 *
 * @param    {Object} [options={}]
 *           A plain object containing options for the plugin.
 */


var onPlayerReady = function onPlayerReady(player, options) {
  player.addClass('vjs-auto-caption');

  if (player.usingPlugin('perSourceBehaviors')) {
    player.on('sourcechanged', function () {
      player.onePerSrc('canplay', update);
    });
  } else {
    player.one('canplay', update);
  }
};
/**
 * A video.js plugin.
 *
 * In the plugin function, the value of `this` is a video.js `Player`
 * instance. You cannot rely on the player being in a "ready" state here,
 * depending on how the plugin is invoked. This may or may not be important
 * to you; if not, remove the wait for "ready"!
 *
 * @function autoCaption
 * @param    {Object} [options={}]
 *           An object of options left to the plugin author to define.
 */


var autoCaption = function autoCaption(options) {
  var _this = this;

  this.ready(function () {
    onPlayerReady(_this, videojs.mergeOptions(defaults, options));
  });
}; // Register the plugin with video.js.


registerPlugin('autoCaption', autoCaption); // Include the version number.

autoCaption.VERSION = version;

export default autoCaption;
